import logging
import re
import os
import traceback
from PyQt5.QtCore import Qt
from PyQt5.QtGui import QPaintDevice
from PyQt5.QtWidgets import QListWidgetItem, QMessageBox

logger = logging.getLogger(__name__)


def longest_common_prefix(*strings):
    end = 0
    for c in zip(*strings):
        if len(set(c)) != 1:
            break
        end += 1
    return strings[0][:end]


def guess_title(*titles):
    result = longest_common_prefix(*titles)
    result = re.fullmatch(r'([\[(].*?[\])])?(.*)', result).group(2)  # Strip leading [] and ()
    result = re.fullmatch(r'[^a-zA-Z0-9]*(.*?)[^a-zA-Z0-9]*', result).group(1)  # Strip non-word characters
    result = result.replace('_', ' ')
    return result


def listwidget_item(text, data):
    """Construct a QListWidgetItem with `text` and UserRole `data`"""
    item = QListWidgetItem()
    item.setText(text)
    item.setData(Qt.UserRole, data)
    return item


def listwidget_text_items(listwidget):
    """Iterator over the 'text's of `listwidget`"""
    for i in range(listwidget.count()):
        yield listwidget.item(i).text()


def listwidget_data_items(listwidget):
    """Iterator over the UserRole 'data's of `listwidget`"""
    for i in range(listwidget.count()):
        yield listwidget.item(i).data(Qt.UserRole)


def noop_action(source, dest):
    """Log action, but don't do anything"""
    logger.info(f"noop_action('{source}', '{dest}')")
    return


def relative_symlink(source, dest):
    """Like symlink, but the link is relative (for portability)"""
    relsource = os.path.relpath(source, os.path.dirname(dest))
    return os.symlink(relsource, dest)


def popup_excepthook(etype, value, tb):
    """For unexpected errors, popup a window with an error message"""
    traceback.print_exception(etype, value, tb)
    while tb is not None:
        parent = tb.tb_frame.f_locals.get('self')
        if isinstance(parent, QPaintDevice):
            break
        tb = tb.tb_next
    else:
        logger.warning('Could not find parent to report exception')
        return
    message = QMessageBox(parent)
    message.setWindowTitle('Error')
    message.setText(str(value))
    message.open()



from PyQt5.QtWidgets import QApplication
from fyler.views.main_window import MainWindow


class FylerApp(QApplication):
    def __init__(self, args_raw):
        super().__init__(args_raw)

    def open_main_window(self):
        self.main_window = MainWindow(self)
        self.main_window.show()
        self.main_window.raise_()



import os
import platform
from importlib import resources

from PyQt5 import uic, QtCore
from PyQt5.QtCore import Qt
from PyQt5.QtWidgets import QFileDialog

from fyler import assets, utils, settings, settings_handler
from fyler.utils import listwidget_text_items, listwidget_data_items, listwidget_item
from fyler.views.advanced_window import AdvancedWindow
from fyler.views.search_window import SearchWindow
from fyler.views.settings_window import SettingsWindow

uifile = resources.open_text(assets, 'mainwindow.ui')
MainWindowUI, MainWindowBase = uic.loadUiType(uifile)


def choose_sources_dialog(parent, title, directory=True):
    options = QFileDialog.Options()
    if directory:
        options |= QFileDialog.ShowDirsOnly
    dialog = QFileDialog(parent, title, os.path.expanduser('~'), options=options)
    dialog.setFileMode(QFileDialog.Directory if directory else QFileDialog.ExistingFiles)
    dialog.setParent(parent, QtCore.Qt.Sheet)
    return dialog


class MainWindow(MainWindowUI, MainWindowBase):
    def __init__(self, parent=None):
        super().__init__()
        self.setupUi(self)

        # Scroll lists together
        self.sourceList.verticalScrollBar().valueChanged.connect(
            self.destList.verticalScrollBar().setValue)
        self.destList.verticalScrollBar().valueChanged.connect(
            self.sourceList.verticalScrollBar().setValue)

        # Select from lists together
        self.sourceList.currentRowChanged.connect(self.destList.setCurrentRow)
        self.destList.currentRowChanged.connect(self.sourceList.setCurrentRow)

        self.folderButton.clicked.connect(lambda: self.add_sources(directory=True))
        self.filesButton.clicked.connect(lambda: self.add_sources(directory=False))

        self.matchButton.clicked.connect(self.match_sources)
        self.actionButton.clicked.connect(self.process_action)
        self.actionButton.setText(settings_handler.action_names[settings['modify_action']])

        self.sourceUpButton.clicked.connect(lambda: self.move_item(self.sourceList, 'up'))
        self.sourceDownButton.clicked.connect(lambda: self.move_item(self.sourceList, 'down'))
        self.destUpButton.clicked.connect(lambda: self.move_item(self.destList, 'up'))
        self.destDownButton.clicked.connect(lambda: self.move_item(self.destList, 'down'))

        self.sourceXButton.clicked.connect(lambda: self.remove_item(self.sourceList))
        self.destXButton.clicked.connect(lambda: self.remove_item(self.destList))

        self.settingsButton.clicked.connect(self.edit_settings)
        self.advancedButton.clicked.connect(self.open_advanced)

    def add_sources(self, directory: bool):
        """Open file selection dialog and load choices into sourceList"""
        def receive():
            self.sourceList.clear()
            self.destList.clear()
            sources = dialog.selectedFiles()
            for item in sources:
                if os.path.isdir(item):
                    for subdir, dirs, files in os.walk(item):
                        for filename in files:
                            filepath = os.path.abspath(os.path.join(subdir, filename))
                            qtitem = listwidget_item(filename, filepath)
                            self.sourceList.addItem(qtitem)
                else:
                    filepath = os.path.abspath(item)
                    qtitem = listwidget_item(os.path.basename(filepath), filepath)
                    self.sourceList.addItem(qtitem)
            self.sourceList.sortItems()

        t = 'directory' if directory else 'files'
        msg = self.tr(f"Choose {t} to rename")
        dialog = choose_sources_dialog(self, msg, directory=directory)
        dialog.open(receive)

    def _update_dest_paths(self):
        """Update the paths based on format and episode data"""
        # Ampherstand chosen since most use cases are for multiple shorts in one episode
        chartab = {ord('/'): '&'}
        if 'windows' in platform.platform().lower():
            chartab.update({
                ord(':'): ' -',
                ord('\\'): None,
                ord('"'): "'",
                ord('<'): None,
                ord('>'): None,
                ord('?'): None,
                ord('*'): '!',
                ord('|'): None,
            })
        for i in range(self.destList.count()):
            qtitem = self.destList.item(i)
            item = qtitem.data(Qt.UserRole)
            # TODO: Formats can fail because of invalid types (sometimes things are None),
            # it would be nice if those cases resulted in a * or something without explicitly
            # handling it in every template_values()
            env = item.template_values()
            for key, value in env.items():
                env[key] = str(value).translate(chartab)
            try:
                dest = settings['output_format'].format(**env)
            except KeyError as e:
                dest = f"*Error: Template variable {e} not found*"
            qtitem.setText(dest)

    def match_sources(self):
        if self.sourceList.count() <= 0:
            return

        # Guess title by removing some noise and taking longest common prefix
        guess = utils.guess_title(*listwidget_text_items(self.sourceList))
        window = SearchWindow(guess)
        window.setParent(self, Qt.Sheet)
        window.show()
        if window.exec_():
            self.destList.clear()
            for item in window.result.items():
                qtitem = listwidget_item('{placeholder}', item)
                self.destList.addItem(qtitem)
            self._update_dest_paths()

    def process_action(self):
        # Loop through rows, but always use top item to delete after its done
        num_actions = min(self.sourceList.count(), self.destList.count())
        for _ in range(num_actions):
            source = self.sourceList.item(0).data(Qt.UserRole)
            dest = os.path.expanduser(os.path.expandvars(self.destList.item(0).text()))
            extension = os.path.splitext(source)[1]
            if destdir := os.path.dirname(dest):
                os.makedirs(destdir, exist_ok=True)
            settings.action()(source, dest + extension)
            self.sourceList.takeItem(0)
            self.destList.takeItem(0)

    def edit_settings(self):
        window = SettingsWindow()
        window.setParent(self, Qt.Sheet)
        window.show()
        window.exec_()
        self.actionButton.setText(settings_handler.action_names[settings['modify_action']])
        self._update_dest_paths()

    def open_advanced(self):
        window = AdvancedWindow()
        window.setParent(self, Qt.Sheet)
        window.show()
        window.exec_()

    def move_item(self, item_list, direction):
        crow = item_list.currentRow()
        if crow < 0:
            return  # Not selected

        # Qt will handle out of bounds selections
        nrow = crow - 1 if direction == 'up' else crow + 1
        item = item_list.takeItem(crow)
        item_list.insertItem(nrow, item)
        item_list.setCurrentRow(nrow)

    def remove_item(self, item_list):
        crow = item_list.currentRow()
        if crow < 0:
            return  # Not selected
        item_list.takeItem(crow)



import csv
import logging
import zlib
from datetime import date
from pathlib import Path

import bs4
import requests
from appdirs import AppDirs
from bs4 import BeautifulSoup
from diskcache import Cache
from fuzzywuzzy import process
from ratelimit import limits, sleep_and_retry

from fyler.models import Media, Series, Episode, Movie
from .provider import Provider

# Redefine dirs since importing settings causes a circular import
dirs = AppDirs('fyler', 'fyler')
_cache_dir = Path(dirs.user_cache_dir) / 'anidb'
_titles_dat = Path(dirs.user_cache_dir) / 'anidb/data/anime-titles.dat'
_cache_dir.mkdir(parents=True, exist_ok=True)
cache = Cache(directory=str(_cache_dir))

logger = logging.getLogger(__name__)


@sleep_and_retry
@limits(calls=2, period=5)
def _rl_get(*args, **kwargs):
    return requests.get(*args, **kwargs)


@cache.memoize(expire=60 * 60 * 24 * 30)  # Cache for 1 month
def _raw_get_info(id: int) -> str:
    args = {
        'request': 'anime',
        'client': 'fyler',
        'clientver': '1',
        'protover': '1',
        'aid': str(id),
    }
    logger.debug('Requesting from AniDB')
    return _rl_get('http://api.anidb.net:9001/httpapi', params=args).text


class AniDBProvider(Provider):
    name = 'AniDB'

    def detail(self, media: Media) -> Media:
        xml = _raw_get_info(media.id)
        soup = BeautifulSoup(xml, 'xml')
        anime = soup.find('anime')

        anime_kwargs = {
            'database': self.name,
            'id': media.id,
            'date': date.fromisoformat(anime.find('startdate').text),
        }
        for title in anime.find('titles').find_all('title'):
            if title.attrs['type'] == 'main':
                anime_kwargs['title'] = title.text
                break

        if anime.find('type').text == 'TV Series':
            episodes = []
            for episode in anime.find('episodes').children:
                if not isinstance(episode, bs4.element.Tag):
                    continue
                if episode.find('epno').attrs['type'] != '1':
                    continue
                episode_kwargs = {
                    'database': 'AniDB',
                    'series': None,  # Will be set later
                    'id': int(episode.attrs['id']),
                    'date': date.fromisoformat(episode.find('airdate').text),
                    'season_number': 0,  # Anime doesn't really do seasons normally...
                    'episode_number': int(episode.find('epno').text),  # I want this to be an int, but sometimes its different (specials?)
                }
                for title in episode.find_all('title'):
                    if title.attrs['xml:lang'] == 'en':
                        episode_kwargs['title'] = title.text
                        break
                episodes.append(Episode(**episode_kwargs))
            episodes.sort(key=lambda x: x.episode_number)
            anime_kwargs['episodes'] = episodes

            s = Series(**anime_kwargs)
            for e in s.episodes:
                e.series = s
            return s
        elif anime.find('type').text == 'Movie':
            return Movie(**anime_kwargs)
        else:
            raise ValueError('Anime is not a TV Series or Movie')

    def download_title_data(self):
        """Download and cache anime titles from AniDB"""
        _titles_dat.parent.mkdir(parents=True, exist_ok=True)
        with open(_titles_dat, 'wb') as dat:
            with requests.get(
                'https://anidb.net/api/anime-titles.dat.gz',
                headers={'User-Agent': 'fyler'},
                stream=True
            ) as response:
                response.raise_for_status()
                z = zlib.decompressobj(wbits=zlib.MAX_WBITS | 16)  # Detect gzip automatically
                for chunk in response.iter_content(chunk_size=8192):
                    dat.write(z.decompress(chunk))
                dat.write(z.flush())

    def _search_by_name(self, query: str) -> list:
        """
        Searches for the closest match to query. Note that this function
        loads the dat file every time, and is not recommended for multiple searches.
        """
        if not _titles_dat.exists():
            logger.info('AniDB title data not found. Downloading fresh copy...')
            self.download_title_data()

        with open(_titles_dat, newline='', encoding='utf-8') as f:
            reader = csv.reader(f, delimiter='|')
            for _ in range(3):
                next(reader)  # Header
            # Filter by primary title
            reader = filter(lambda k: int(k[1]) == 1, reader)
            return [k[0] for k in process.extract(query, reader, limit=10)]

    def search(self, query: str) -> list:
        return [
            Media(
                database=self.name,
                title=k[3],
                id=int(k[0]),
                date=None,
            )
            for k in self._search_by_name(query)
        ]



import logging
from datetime import date
from pathlib import Path

import requests
from appdirs import AppDirs
from diskcache import Cache

from fyler.models import Media, Series, Episode
from .provider import Provider

# Redefine dirs since importing settings causes a circular import
dirs = AppDirs('fyler', 'fyler')
_cache_dir = Path(dirs.user_cache_dir) / 'thetvdb'
_cache_dir.mkdir(parents=True, exist_ok=True)
cache = Cache(directory=str(_cache_dir))

API_ROOT = 'https://api.thetvdb.com'
API_KEY = '2212209a47d8300a7b5c292b8619c7a3'

logger = logging.getLogger(__name__)


@cache.memoize(expire=60 * 60 * 24)  # Tokens are valid for 24 hours
def _get_jwt_token() -> str:
    response = requests.post(f'{API_ROOT}/login', json={'apikey': API_KEY})
    response.raise_for_status()
    return response.json()['token']


class TheTVDBProvider(Provider):
    name = 'TheTVDB'

    def detail(self, media: Media) -> Media:
        response = requests.get(
            f'{API_ROOT}/series/{media.id}',
            headers={'Authorization': f'Bearer {_get_jwt_token()}'}
        )
        response.raise_for_status()
        parsed = response.json()['data']
        series = Series(
            database=self.name,
            id=media.id,
            date=date.fromisoformat(parsed['firstAired']),  # TODO
            title=parsed['seriesName'],
            episodes=None
        )

        last = 0
        page = 1
        episodes = []
        while not last:
            response = requests.get(
                f'{API_ROOT}/series/{media.id}/episodes',
                params={'page': page},
                headers={'Authorization': f'Bearer {_get_jwt_token()}'}
            )
            response.raise_for_status()

            for episode in response.json()['data']:
                episodes.append(Episode(
                    database=self.name,
                    id=episode['id'],
                    title=episode['episodeName'],
                    season_number=episode['airedSeason'],
                    episode_number=episode['airedEpisodeNumber'],
                    date=date.fromisoformat(episode['firstAired']),  # TODO
                    series=series,
                ))

            last = response.json()['links']['last']
            page = response.json()['links']['next']

        series.episodes = episodes
        return series

    def search(self, query: str) -> list:
        response = requests.get(
            f'{API_ROOT}/search/series',
            params={'name': query},
            headers={'Authorization': f'Bearer {_get_jwt_token()}'}
        )
        response.raise_for_status()
        return [
            Media(
                database=self.name,
                title=k['seriesName'],
                id=k['id'],
                date=None,
            )
            for k in response.json()['data']
        ]



from importlib import resources
from PyQt5 import uic
from fyler import providers, assets

uifile = resources.open_text(assets, 'advancedwindow.ui')
AdvancedWindowUI, AdvancedWindowBase = uic.loadUiType(uifile)


class AdvancedWindow(AdvancedWindowUI, AdvancedWindowBase):
    """Window for Advanced Tools, usually special action that don't happen often"""
    def __init__(self):
        super().__init__()
        self.setupUi(self)
        self.anidbDownloadButton.clicked.connect(self.download_anidb_data)

    def download_anidb_data(self):
        providers.all_providers['anidb'].download_title_data()



import os
import json
from collections import UserDict
from appdirs import AppDirs

from fyler.providers import all_providers
from fyler.utils import noop_action, relative_symlink

dirs = AppDirs('fyler', 'fyler')
CONFIG_FILE = os.environ.get('FYLER_CONFIG_FILE', os.path.join(dirs.user_config_dir, 'config.json'))

# internal name -> func
_action_funcs = {
    'rename': os.rename,
    'absolute_symlink': os.symlink,
    'relative_symlink': relative_symlink,
    'hardlink': os.link,
    'noop': noop_action,
}

# internal name -> friendly name
# TODO: one dict? More confusing but no duplication
action_names = {
    'rename': 'Rename',
    'absolute_symlink': 'Absolute Symlink',
    'relative_symlink': 'Relative Symlink',
    'hardlink': 'Hardlink',
    'noop': 'No-op (Debug)',
}


class SettingsDict(UserDict):
    appdirs = dirs

    def provider(self):
        return all_providers[self['provider']]

    def action(self):
        return _action_funcs[self['modify_action']]


def default_settings():
    return SettingsDict({
        'provider': 'thetvdb',
        'modify_action': 'rename',
        'output_format': '{n} - {s00e00} - {t}',
        'search_result_format': '{t}',
    })


def load_settings(filepath=CONFIG_FILE):
    """Load settings from `filepath`"""
    ret = default_settings()
    try:
        with open(filepath) as f:
            ret.update(json.load(f))
    except Exception:
        pass
    return ret


settings = load_settings()


def save_settings(filepath=CONFIG_FILE):
    """Save settings to `filepath`"""
    if not os.path.exists(CONFIG_FILE):
        os.makedirs(os.path.dirname(CONFIG_FILE), mode=0o775, exist_ok=True)
    with open(CONFIG_FILE, 'w') as f:
        json.dump(settings.data, f, indent=4)



from importlib import resources

from PyQt5 import uic

from fyler import settings, settings_handler, providers, assets

uifile = resources.open_text(assets, 'settingswindow.ui')
SettingsWindowUI, SettingsWindowBase = uic.loadUiType(uifile)


class SettingsWindow(SettingsWindowUI, SettingsWindowBase):
    def __init__(self):
        super().__init__()
        self.setupUi(self)
        self.load_settings()
        self.saveButton.clicked.connect(self.save_settings)

    def load_settings(self):
        current = 0
        for idx, (key, value) in enumerate(providers.all_providers.items()):
            self.providerBox.addItem(value.name, key)
            if key == settings['provider']:
                current = idx
        self.providerBox.setCurrentIndex(current)

        current = 0
        for idx, (key, value) in enumerate(settings_handler.action_names.items()):
            self.actionBox.addItem(value, key)
            if key == settings['modify_action']:
                current = idx
        self.actionBox.setCurrentIndex(current)

        self.formatEdit.setText(settings['output_format'])
        self.searchEdit.setText(settings['search_result_format'])

    def save_settings(self):
        newsettings = {
            'provider': self.providerBox.currentData(),
            'modify_action': self.actionBox.currentData(),
            'output_format': self.formatEdit.text(),
            'search_result_format': self.searchEdit.text(),
        }
        settings.update(newsettings)
        settings_handler.save_settings()
        self.accept()



from datetime import date

import requests

from fyler.models import Series
from .provider import Provider

API_ROOT = 'https://graphql.anilist.co/'


class AniListProvider(Provider):
    name = "AniList (Doesn't work. No episode data)"

    def detail(self, series: Series) -> Series:
        """AniList doesn't have episode data, so nothing to add"""
        return series

    def search(self, query: str) -> list:
        search_gql = '''
            query ($query: String) {
              Page {
                media(search: $query, type: ANIME) {
                  id
                  title {
                    romaji
                    english
                    native
                  }
                  startDate {
                    year
                    month
                    day
                  }
                }
              }
            }
        '''
        body = {
            'query': search_gql,
            'variables': {'query': query}
        }
        response = requests.post(API_ROOT, json=body)
        response.raise_for_status()
        ret = []
        for result, _ in zip(response.json()['data']['Page']['media'], range(5)):
            ret.append(
                Series(
                    database=self.name,
                    title=result['title']['romaji'],
                    id=result['id'],
                    date=date(
                        int(result['startDate']['year']),
                        int(result['startDate']['month']),
                        int(result['startDate']['day']),
                    ),
                    episodes=[],
                ))

        return ret



import logging
import sys
from argparse import ArgumentParser
from fyler.application import FylerApp
from fyler.utils import popup_excepthook


def parse_args():
    parser = ArgumentParser(
        description='A GUI tool to bulk rename files based on metadata from web databases'
    )
    parser.add_argument('-v', '--verbose', action='store_true')
    return parser.parse_args()


def main():
    args = parse_args()
    logging.basicConfig(level=logging.DEBUG if args.verbose else logging.INFO)
    sys.excepthook = popup_excepthook
    app = FylerApp(sys.argv)
    app.open_main_window()
    app.exec_()


if __name__ == '__main__':
    main()



#!/usr/bin/env python3.8
from setuptools import setup, find_packages

# Package meta-data.
DESCRIPTION = 'An application to help organize TV Shows/Anime/Movies'
REQUIRED = [
    # UI
    'PyQT5',

    # AniDB
    'fuzzywuzzy[speedup]',
    'ratelimit',
    'beautifulsoup4',
    'lxml',

    # General
    'requests',
    'appdirs',
    'diskcache',
]

setup(
    name='fyler',
    version='0.1.0',
    description=DESCRIPTION,
    long_description=DESCRIPTION,
    author='Alex Gonzales<musigonzales@gmail.com>',
    python_requires='>=3.8.0',
    packages=find_packages(exclude=['tests']),
    package_data={'fyler.assets': ['*.ui']},
    entry_points={
        'gui_scripts': ['fyler=fyler.__main__:main'],
        'fyler.providers': [
            'anidb = fyler.providers.anidb:AniDBProvider',
            'anilist = fyler.providers.anilist:AniListProvider',
            'thetvdb = fyler.providers.thetvdb:TheTVDBProvider',
        ]
    },
    install_requires=REQUIRED
)



from fyler.models import Media


class Provider:
    def detail(self, media: Media) -> Media:
        """Add more detail to a search result"""
        raise NotImplementedError()

    def search(self, query: str) -> list:
        """Search for a query and return a list of Media results"""
        raise NotImplementedError()



import dataclasses
from dataclasses import dataclass

from datetime import date


@dataclass
class Media:
    id: int
    database: str
    title: str
    date: date

    # I wish dataclasses had this interface naturally,
    # it's a bit more convenient imo.
    def asdict(self):
        return dataclasses.asdict(self)

    def astuple(self):
        return dataclasses.astuple(self)

    def items(self):
        """Returns an iterable for things to rename to"""
        raise NotImplementedError()

    def template_values(self) -> dict:
        return {
            'id': self.id,
            'db': self.database,
            't': self.title,
            'y': self.date.year if self.date else '*',
        }


@dataclass
class Series(Media):
    episodes: list

    def items(self):
        return self.episodes


@dataclass
class Episode(Media):
    series: Series
    episode_number: int
    season_number: int

    def template_values(self) -> dict:
        return dict(**super().template_values(), **{
            'n': self.series.title,
            's': self.season_number,
            'e': self.episode_number,
            'sxe': f'{self.season_number}x{self.episode_number:02}',
            's00e00': f'S{self.season_number:02}E{self.episode_number:02}' if self.season_number and self.episode_number else '*',
            'e00': f'{self.episode_number:02}',
        })


@dataclass
class Movie(Media):
    def items(self):
        return [self]



from importlib import resources

from PyQt5 import uic
from PyQt5.QtCore import Qt

from fyler import utils, settings, assets

uifile = resources.open_text(assets, 'searchwindow.ui')
SearchWindowUI, SearchWindowBase = uic.loadUiType(uifile)


class SearchWindow(SearchWindowUI, SearchWindowBase):
    def __init__(self, initial_query=None):
        super().__init__()
        self.setupUi(self)
        self.result = None

        # TODO: Consider changing the search text to something like "Search ({provider})"
        self.searchButton.clicked.connect(self.do_search)
        self.searchButton.setText(f'Search {settings.provider().name}')
        self.resultList.itemDoubleClicked.connect(self.accept_result)

        if initial_query:
            self.searchBox.setText(initial_query)

    def accept_result(self):
        self.result = self.resultList.currentItem().data(Qt.UserRole)
        self.result = settings.provider().detail(self.result)
        self.accept()

    def do_search(self):
        if not self.searchBox.text():
            return

        self.resultList.clear()
        results = settings.provider().search(self.searchBox.text())

        for item in results:
            try:
                text = settings['search_result_format'].format(**item.template_values())
            except KeyError as e:
                text = f"*Error: Template variable {e} not found*"
            qtitem = utils.listwidget_item(text, item)
            self.resultList.addItem(qtitem)
